package com.flutter.uninstall.flutter_uninstall;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.net.Uri;

import androidx.annotation.NonNull;

import java.util.ArrayList;
import java.util.List;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

/** FlutterUninstallPlugin */
public class FlutterUninstallPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel channel;
  private Activity activity;

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "flutter_uninstall");
    channel.setMethodCallHandler(this);
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    if (call.method.equals("getPlatformVersion")) {
      result.success("Android " + android.os.Build.VERSION.RELEASE);
    } else if(call.method.equals("getVersionName")){
      String packageName = call.argument("packageName");
      if(activity!=null && packageName!=null){
        String versionName = getVersionName(activity, packageName);
        result.success(versionName);
      }else{
        result.success(null);
      }
    } else if(call.method.equals("getVersionCode")){
      String packageName = call.argument("packageName");
      if(activity!=null && packageName!=null){
        int versionCode = getVersionCode(activity, packageName);
        result.success(versionCode);
      }else{
        result.success(0);
      }
    } else if(call.method.equals("getAppLabel")){
      String packageName = call.argument("packageName");
      if(activity!=null && packageName!=null){
        String label = getAppLabel(activity, packageName);
        result.success(label);
      }else{
        result.success(null);
      }
    } else if(call.method.equals("getPackagesByLabel")){
      String packageName = call.argument("label");
      if(activity!=null && packageName!=null){
        List<String> packages = getPackagesByLabel(activity, packageName);
        result.success(packages);
      }else{
        result.success(null);
      }
    } else if(call.method.equals("isInstallApp")){
      String packageName = call.argument("packageName");
      if(activity!=null && packageName!=null){
        boolean install = isAppInstalled(activity, packageName);
        result.success(install);
      }else{
        result.success(false);
      }
    } else if(call.method.equals("uninstallApp")){
      String packageName = call.argument("packageName");
      if(activity!=null && packageName!=null){
        uninstall(activity, packageName);
      }
      result.success(null);
    } else {
      result.notImplemented();
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }

  @Override
  public void onAttachedToActivity(@NonNull ActivityPluginBinding binding){
    this.activity = binding.getActivity();
  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {

  }

  @Override
  public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
    this.activity = binding.getActivity();
  }

  @Override
  public void onDetachedFromActivity() {

  }

  private String getVersionName(Context context, String packageName){
    final PackageManager packageManager = context.getPackageManager();
    try {
      PackageInfo packageInfo = packageManager.getPackageInfo(packageName, 0);
      return packageInfo.versionName;
    }catch (PackageManager.NameNotFoundException e){
      e.printStackTrace();
    }
    return null;
  }

  private int getVersionCode(Context context, String packageName){
    final PackageManager packageManager = context.getPackageManager();
    try {
      PackageInfo packageInfo = packageManager.getPackageInfo(packageName, 0);
      return packageInfo.versionCode;
    }catch (PackageManager.NameNotFoundException e){
      e.printStackTrace();
    }
    return 0;
  }

  private String getAppLabel(Context context, String packageName){
    final PackageManager packageManager = context.getPackageManager();
    try {
      ApplicationInfo appInfo = packageManager.getApplicationInfo(packageName, PackageManager.GET_META_DATA);
      CharSequence appLabel =  packageManager.getApplicationLabel(appInfo);
      return appLabel.toString();
    }catch (PackageManager.NameNotFoundException e){
      e.printStackTrace();
    }
    return null;
  }

  private List<String> getPackagesByLabel(Context context, String label){
    final PackageManager packageManager = context.getPackageManager();
    List<PackageInfo> pinfo = packageManager.getInstalledPackages(0);
    List<String> pName = new ArrayList<>();
    if (pinfo != null) {
      for (int i = 0; i < pinfo.size(); i++) {
        String pn = pinfo.get(i).packageName;
        String lb = this.getAppLabel(context, pn);
        if(label.equals(lb)){
          pName.add(pn);
        }
      }
    }
    return pName;
  }

  private boolean isAppInstalled(Context context, String packageName) {
    final PackageManager packageManager = context.getPackageManager();
    List<PackageInfo> packInfo = packageManager.getInstalledPackages(0);
    List<String> pName = new ArrayList<>();
    if (packInfo != null) {
      for (int i = 0; i < packInfo.size(); i++) {
        String pn = packInfo.get(i).packageName;
        pName.add(pn);
      }
    }
    return pName.contains(packageName);
  }

  private void uninstall(Activity activity, String packageName){
    Uri packageUri = Uri.parse("package:"+packageName);
    Intent intent = new Intent(Intent.ACTION_DELETE,packageUri);
    intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
    activity.startActivity(intent);
  }
}
